searchData={"items":[{"type":"module","title":"cuckoo_filter","doc":"High-performance, concurrent, and mutable Cuckoo Filter implemented using atomics for Erlang and Elixir.","ref":"cuckoo_filter.html"},{"type":"function","title":"cuckoo_filter.add/2","doc":"Equivalent to add(Filter, Element, false) .","ref":"cuckoo_filter.html#add/2"},{"type":"function","title":"cuckoo_filter.add/3","doc":"Adds an element to a filter. Returns ok if the insertion was successful, but could return {error, not_enough_space} , when the filter is nearing its capacity. Forced argument is a boolean that indicates whether the insertion should be forced or not. A forced insertion will never return a not_enough_space error, instead when the filter is full, another random element is removed, and the removed element is returned as {ok, {Index, Fingerprint}} . In this case, elements are not relocated, and no lock is acquired. Forced insertion can only be used with max_evictions set to 0.","ref":"cuckoo_filter.html#add/3"},{"type":"function","title":"cuckoo_filter.add_hash/2","doc":"Equivalent to add_hash(Filter, Element, false) .","ref":"cuckoo_filter.html#add_hash/2"},{"type":"function","title":"cuckoo_filter.add_hash/3","doc":"Adds an element to a filter by its hash. Same as add/3 except that it accepts the hash of the element instead of the element.","ref":"cuckoo_filter.html#add_hash/3"},{"type":"function","title":"cuckoo_filter.capacity/1","doc":"Returns the maximum capacity of a filter.","ref":"cuckoo_filter.html#capacity/1"},{"type":"function","title":"cuckoo_filter.contains/2","doc":"Checks if an element is in a filter.","ref":"cuckoo_filter.html#contains/2"},{"type":"function","title":"cuckoo_filter.contains_fingerprint/3","doc":"Checks whether a filter contains a fingerprint at the given index or its alternative index.","ref":"cuckoo_filter.html#contains_fingerprint/3"},{"type":"function","title":"cuckoo_filter.contains_hash/2","doc":"Checks if an element is in a filter by its hash.","ref":"cuckoo_filter.html#contains_hash/2"},{"type":"function","title":"cuckoo_filter.delete/2","doc":"Deletes an element from a filter. Returns ok if the deletion was successful, and returns {error, not_found} if the element could not be found in the filter. Note: A cuckoo filter can only delete items that are known to be inserted before. Deleting non inserted items might lead to deletion of another element in case of a hash collision.","ref":"cuckoo_filter.html#delete/2"},{"type":"function","title":"cuckoo_filter.delete_hash/2","doc":"Deletes an element from a filter by its hash. Same as delete/2 except that it uses the hash of the element instead of the element.","ref":"cuckoo_filter.html#delete_hash/2"},{"type":"function","title":"cuckoo_filter.export/1","doc":"Exports a filter as a binary. Returned binary can be used to reconstruct the filter again, using import/2 function.","ref":"cuckoo_filter.html#export/1"},{"type":"function","title":"cuckoo_filter.hash/2","doc":"Returns the hash value of an element using the hash function of the filter.","ref":"cuckoo_filter.html#hash/2"},{"type":"function","title":"cuckoo_filter.import/2","doc":"Imports filter data from a binary created using export/1 . Returns ok if the import was successful, but could return {ok, invalid_data_size} if the size of the given binary does not match the size of the filter.","ref":"cuckoo_filter.html#import/2"},{"type":"function","title":"cuckoo_filter.new/1","doc":"Equivalent to new(Capacity, []) .","ref":"cuckoo_filter.html#new/1"},{"type":"function","title":"cuckoo_filter.new/2","doc":"Creates a new cuckoo filter with the given capacity and options Note that the actual capacity might be higher than the given capacity, because internally number of buckets in a cuckoo filter must be a power of 2. Possible options are: {name, Name} If you give it a name, created filter instance will be stored in persistent_term, and later you can access the filter by its name. {fingerprint_size, FingerprintSize} FingerprintSize can be one of 4, 8, 16, 32, and 64 bits. Default fingerprint size is 16 bits. {bucket_size, BucketSize} BucketSize must be a non negative integer, and the default value is 4. Higher bucket sizes can reduce insert time considerably since it reduces the number of relocations of existing fingerprints in occupied buckets, but it increases the lookup time, and false positive rate. {max_evictions, MaxEvictions} MaxEvictions indicates the maximum number of relocation attemps before giving up when inserting a new element. {hash_function, HashFunction} You can specify a custom hash function that accepts a binary as argument and returns hash value as an integer. By default xxh3 hash functions are used, and you need to manually add xxh3 to the list of your project dependencies.","ref":"cuckoo_filter.html#new/2"},{"type":"function","title":"cuckoo_filter.size/1","doc":"Returns number of items in a filter.","ref":"cuckoo_filter.html#size/1"},{"type":"function","title":"cuckoo_filter.whereis/1","doc":"Retrieves a cuckoo_filter from persistent_term by its name.","ref":"cuckoo_filter.html#whereis/1"},{"type":"type","title":"cuckoo_filter.cuckoo_filter/0","doc":null,"ref":"cuckoo_filter.html#t:cuckoo_filter/0"},{"type":"type","title":"cuckoo_filter.filter_name/0","doc":null,"ref":"cuckoo_filter.html#t:filter_name/0"},{"type":"type","title":"cuckoo_filter.fingerprint/0","doc":null,"ref":"cuckoo_filter.html#t:fingerprint/0"},{"type":"type","title":"cuckoo_filter.hash/0","doc":null,"ref":"cuckoo_filter.html#t:hash/0"},{"type":"type","title":"cuckoo_filter.index/0","doc":null,"ref":"cuckoo_filter.html#t:index/0"},{"type":"type","title":"cuckoo_filter.option/0","doc":null,"ref":"cuckoo_filter.html#t:option/0"},{"type":"type","title":"cuckoo_filter.options/0","doc":null,"ref":"cuckoo_filter.html#t:options/0"},{"type":"extras","title":"Cuckoo Filter","doc":"# Cuckoo Filter\n\n[![CI build status](https://github.com/farhadi/cuckoo_filter/workflows/CI/badge.svg)](https://github.com/farhadi/cuckoo_filter/actions?query=workflow%3ACI)\n[![codecov](https://codecov.io/gh/farhadi/cuckoo_filter/branch/main/graph/badge.svg)](https://codecov.io/gh/farhadi/cuckoo_filter)\n[![Hex docs](http://img.shields.io/badge/hex.pm-docs-green.svg?style=flat)](https://hexdocs.pm/cuckoo_filter)\n[![Hex Version](http://img.shields.io/hexpm/v/cuckoo_filter.svg?style=flat)](https://hex.pm/packages/cuckoo_filter)\n[![License](http://img.shields.io/hexpm/l/cuckoo_filter.svg?style=flat)](https://github.com/farhadi/cuckoo_filter/blob/master/LICENSE)\n\nA high-performance, concurrent, and mutable [Cuckoo Filter](https://www.cs.cmu.edu/~dga/papers/cuckoo-conext2014.pdf)\nimplemented using [atomics](https://erlang.org/doc/man/atomics.html) for Erlang and Elixir.","ref":"readme.html"},{"type":"extras","title":"Introduction - Cuckoo Filter","doc":"A Cuckoo Filter is a space-efficient probabilistic data structure for approximated\nset-membership queries. It can be used to test whether an element is a member of a set in\nconstant time and requires only a few bits per element. The trade-off is that a low rate\nof false positives is possible, but false negatives are not.\n\nCuckoo Filter is considered a better alternative to Bloom Filter with lower space\noverhead and with support for deletion of inserted elements.\n\nInsertion of elements in a cuckoo filter becomes slowers when the load factor is high and\nmight fail when capacity is nearly full.","ref":"readme.html#introduction"},{"type":"extras","title":"Implementation - Cuckoo Filter","doc":"In this implementation, filter data is stored in an atomics array, which is a fixed-size\nmutable array of 64-bit integers. By using atomics we can have fast and concurrent\naccess to the filter for both reads and writes.\n\nTo be able to update fingerprints atomically, this implementation only allows fingerprint\nsizes of 4, 8, 16, 32, and 64 bits, so that multiple fingerprints can fit in a single\n64-bit atomic integer.\n\nIn a Cuckoo Filter for each element, there are two buckets where it can be inserted. To\ninsert an element when there is an empty slot available in one of the two buckets, we\ncan atomically update that empty entry, but when there is no slot available in the two\nbuckets we have to relocate exiting elements to make room for the new one. In this case,\nsince multiple entries need to be updated, we no longer can do insert operation\natomically, and such inserts can not be done concurrently. In such cases, to prevent\nrace conditions we use the first integer in the atomics array as a mutex.\n\nDeletes are also done with a lock to prevent race conditions when an insert is relocating\nelements.\n\nWhen relocating existing elements, it is also possible that a concurrent lookup of an\nevicted element fails. To prevent this, instead of immediately updating an evicted\nelement, we keep a sequence of evictions in memory, and once we find an empty slot we\nstart applying the changes in the sequence from the end to the beginning. In other words,\nto relocate an element, we first insert it in its new place, then we remove it from its\nold place, making sure the element is always available to lookups.\n\nIn most cuckoo filter implementations, when an insert fails, the element is actually\nadded to the filter, but some other random element gets removed, but in this\nimplementation, by using this eviction cache technique we can also avoid the removal\nof a random element when an insertion fails.\n\nAnother benefit of keeping the chain of evictions in memory is the early detection of\nloops before reaching the maximum number of evictions.\n\nThe second integer in the array is used as an atomic counter to store the number of\nelements inserted in the filter. \n\nFor hashing, by default 64-bit variant of [XXH3](https://github.com/farhadi/xxh3) is used.\n\nFingerprint size, bucket size, hash function, and the maximum number of evictions are\nconfigurable when creating a new filter.","ref":"readme.html#implementation"},{"type":"extras","title":"Usage - Cuckoo Filter","doc":"If you want to use the default xxh3 hash functions, you need to add\n[xxh3](https://hex.pm/packages/xxh3) to your deps.\n\nIn Erlang\n\n```erlang\nFilter = cuckoo_filter:new(1000),\nok = cuckoo_filter:add(Filter, 1),\ntrue = cuckoo_filter:contains(Filter, 1),\nfalse = cuckoo_filter:contains(Filter, 2),\nok = cuckoo_filter:delete(Filter, 1),\n{error, not_found} = cuckoo_filter:delete(Filter, 1).\n```\n\nIn Elixir\n\n```elixir\nfilter = :cuckoo_filter.new(1000)\n:ok = :cuckoo_filter.add(filter, 1)\ntrue = :cuckoo_filter.contains(filter, 1)\nfalse = :cuckoo_filter.contains(filter, 2)\n:ok = :cuckoo_filter.delete(filter, 1)\n{:error, :not_found} = :cuckoo_filter.delete(filter, 1)\n```\n\nFor more details, see the module documentation.","ref":"readme.html#usage"},{"type":"extras","title":"License - Cuckoo Filter","doc":"Copyright 2021, Ali Farhadi  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   Copyright 2021, Ali Farhadi  .\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/plain"}